#ifndef _I2C_MASTER_FUNCTIONS_H_
	#define _I2C_MASTER_FUNCTIONS_H_


	#define SCL_HP			23
	#define SCL_LP			22
	#define i2c_pin_config	PINSEL1|=((1<<22)|(1<<24))

	#define false			0
	#define true			1



volatile unsigned char* buff;		//buffer
volatile unsigned int counter=0;	
volatile unsigned char i2c_busy=false;
volatile unsigned char i2c_error=false;
volatile unsigned char sla=0x00;

__irq void i2c0_isr(void)
{	
	switch (I20STAT & 0xF8)
	{
		case 0x08 :					//a start condition has been transmitted
			
		case 0x10 :				  	//a repeated start condition has been transmitted
			I20DAT = sla;		 	//slave address + WR
			I20CONCLR = (1<<5);	 	//clear STA
			break;
		case 0x18 :			   		//SLA+W has been transmitted,ACK has been received
			if(counter==0)
			{
				I20CONSET=(1<<4);	//stop condition will be transmitted
				i2c_busy=false;	
			}
			else
			{
				I20DAT = *buff;		//first data byte will be transmitted
				buff++;
				counter--;
			}
			break;
		case 0x20 :		   			//SLA+W has been transmitted,NOTACK has been received
			I20CONSET=(1<<4);	 	//stop condition will be transmitted
			i2c_busy=false;		   	
			i2c_error=true;
			break;
		case 0x28 :			  		//data byte in I2DAT has been transmitted,ACK has been received
			if(counter)			 	
			{
				I20DAT = *buff;		//transmiting next data byte
				buff++;
				counter--;
			}
			else				 	//last data byte
			{
				I20CONSET=(1<<4);	//stop condition
				i2c_busy=false;
			}
			break;
		case 0x30 :			  		//data byte in I2DAT has been transmitted,NOTACK has been received
			I20CONSET=(1<<4);		//stop condition
			i2c_busy=false;
			i2c_error=true;	
			break;
		case 0x40 :		 			//SLA+R has been transmitted,ACK has been received
			if(counter>1)
				I20CONSET=(1<<2);	//ACK will be returned for next received data byte
			else
				I20CONCLR=(1<<2);	//NOTACK will be returned for next received data byte
			break;
		case 0x48 :				  	//SLA+R has been transmitted,NOTACK has been received
			I20CONSET=(1<<4);  		//stop condition
			i2c_busy=false;	
			i2c_error=true;
			break;
		case 0x50 :					//data byte has been received,ACK has been returned			
			*buff++=I20DAT;		 	//buffering received data byte
			counter--;
			if(counter>1)			
				I20CONSET=(1<<2);	//ACK will be returned for next received data byte
			else
				I20CONCLR=(1<<2);	//NOTACK will be returned for next received data byte
			
			break;
		case 0x58 :		  			//data byte has been received,NOTACK has been returned
			*buff=I20DAT;	 		//buffering received data byte
			I20CONSET=(1<<4);		//stop condition
			i2c_busy=false;	
			break;
		default :
			i2c_busy=false;	
			i2c_error=true;
	}	

	I20CONCLR = (1<<3);				//clear SI bit
	VICVectAddr=0x00000000;	   		
}

void i2c0_init(void)
{
	VICVectAddr9=(unsigned long)i2c0_isr;
	VICIntEnable=(1<<9);  			//enable i2c0 interrupt
	i2c_pin_config; 
	I20CONCLR=0x2C;					//i2c0 reset
	I20SCLL = SCL_LP;		   		//config duty cycle & frequency
	I20SCLH = SCL_HP;		   		//config duty cycle & frequency
	I20CONSET=(1<<6);				//enable i2c0
}

unsigned char i2c_tranceive_data(unsigned char slave_address,unsigned char data_read,unsigned int data_length,unsigned char* pointer)
{
	
	i2c_busy=true;
	i2c_error=false;
	sla=(slave_address<<1);
	if(data_read)sla++;
	counter=data_length;
	buff=pointer;			   
	I20CONSET=(1<<5);				//set STA : start condition
	while(i2c_busy);
	if(i2c_error==true) 
		return(false);
		
	return(true);
}

#endif
